@@ -16,12 +16,17 @@ use Class::Load 0 ();
 	
 	BEGIN {
 		$MooseX::Interface::AUTHORITY = 'cpan:TOBYINK';
-		$MooseX::Interface::VERSION   = '0.001';
+		$MooseX::Interface::VERSION   = '0.002';
 	
 		*requires = \&Moose::Role::requires;
 		*excludes = \&Moose::Role::excludes;
 	}
 	
+	sub test_case (&;$)
+	{
+		Class::MOP::class_of( (scalar caller)[0] )->add_test_case(@_);
+	}
+	
 	sub const
 	{
 		my ($meta, $name, $value) = @_;
@@ -39,6 +44,7 @@ use Class::Load 0 ();
 
 	my ($import, $unimport) = Moose::Exporter->build_import_methods(
 		with_meta => [qw( extends excludes const requires )],
+		as_is     => [qw( test_case )],
 	);
 	
 	sub unimport
@@ -76,18 +82,35 @@ use Class::Load 0 ();
 	package MooseX::Interface::Trait::Method::Constant;
 	use Moose;
 	extends 'Moose::Meta::Method';
+	BEGIN {
+		$MooseX::Interface::Trait::Method::Constant::AUTHORITY = 'cpan:TOBYINK';
+		$MooseX::Interface::Trait::Method::Constant::VERSION   = '0.002';
+	}
 }
 
 {
 	package MooseX::Interface::Trait::Role;
 	use Moose::Role;
+	use Contextual::Return;
+	use namespace::clean;
+	
+	BEGIN {
+		$MooseX::Interface::Trait::Role::AUTHORITY = 'cpan:TOBYINK';
+		$MooseX::Interface::Trait::Role::VERSION   = '0.002';
+	}
 
-	has 'is_interface' => (
+	has is_interface => (
 		is      => 'rw',
 		isa     => 'Bool',
 		default => 0,
 	);
 	
+	has test_cases => (
+		is      => 'ro',
+		isa     => 'ArrayRef',
+		default => sub { [] },
+	);
+	
 	sub add_constant
 	{
 		my ($meta, $name, $value) = @_;
@@ -99,6 +122,36 @@ use Class::Load 0 ();
 			),
 		);
 	}
+	
+	sub add_test_case
+	{
+		my ($meta, $coderef, $name) = @_;
+		$name //= sprintf("Test case %d", @{ $meta->test_cases } + 1);
+		push @{ $meta->test_cases }, [$coderef, $name];
+	}
+	
+	sub test_implementation
+	{
+		my ($meta, $instance) = @_;
+		confess("Parameter is not an object that implements the interface; died")
+			unless blessed($instance) && $instance->DOES($meta->name);
+		
+		my @failed;
+		foreach my $case ( @{ $meta->test_cases } )
+		{
+			my ($code, $name) = @$case;
+			local $_ = $instance;
+			push @failed, $name unless $code->();
+		}
+		
+		return
+			LIST     { @failed }
+			BOOL     { @failed ? 0 : 1 }
+			NUM      { scalar @failed }
+			STR      { @failed ? 'not ok' : 'ok' }
+			ARRAYREF { \@failed }
+		;
+	}
 
 	sub find_problematic_methods
 	{
@@ -257,6 +310,55 @@ method signatures.
 Experimental syntactic sugar for declaring constants. It's probably not a
 good idea to use this yet.
 
+=item C<< test_case { BLOCK } $name >>
+
+Experimental syntactic sugar for embedded test cases. This extends the idea
+that an interface is a contract for classes to fulfil.
+
+The block will be called with an instance of a class claiming to implement
+the interface in C<< $_ >> and should return true if the instance passes the
+test and false if it fails.
+
+  package CalculatorAPI
+  {
+    use MooseX::Interface;
+    
+    requires 'add';
+    test_case { $_->add(8, 2) == 10 };
+    
+    requires 'subtract';
+    test_case { $_->subtract(8, 2) == 6 };
+    
+    requires 'multiply';
+    test_case { $_->multiply(8, 2) == 16 };
+    
+    requires 'divide';
+    test_case { $_->divide(8, 2) == 4 };
+  }
+  
+  package Calculator
+  {
+    use Moose;
+    with 'CalculatorAPI';
+    sub add      { $_[1] + $_[2] }
+    sub subtract { $_[1] - $_[2] }
+    sub multiply { $_[1] * $_[2] }
+    sub divide   { $_[1] / $_[2] }
+  }
+  
+  my $result = CalculatorAPI->meta->test_implementation(
+    Calculator->new,
+  );
+
+The result of C<test_implementation> is a L<Contextual::Return> object which
+indicates success when evaluated in boolean context; indicates the number of
+failures in numeric context; and provides TAP-like "ok" or "not ok" in
+string context.
+
+Do not rely on test cases being run in any particular order, or maintaining
+any state between test cases. (Theoretically each test case could be run with
+a separate instance of the implementing class.)
+
 =back
 
 =begin private
